/*

    P O W E R   P A C K E D   P R I N T O U T S
    Copyright (c) 1994
    Stephen L. Woolstenhulme
    All Rights Reserved

    CIS:  73060,1302

*/

// PPPdemo.prg:   demo program turns default directory into a report.

#include "PPPtree.ch"
#include "inkey.ch"

function pppDemo()
   local nChoice := 0
   local nLeft := int( maxcol()/2-30 )
   local nRight := int( maxcol()/2+30 )
   local nLen := nRight - nLeft + 1
   
   pushscrn()                                    // push screen onto the stack
   setcolor( 'w+/rb, n/w,,, w+/n' )              // and set our own color.
   cls
   setcolor( 'w+/b, n/w,,, w+/n' )               // now set a contract color.
   
   @  5, nLeft, 12, nRight box B_SOLID_TOP       // see ppptree.ch
   @  6, nLeft say padc( 'Power Packed Printouts', nLen )
   @  7, nLeft say replicate( '', nLen )
   @  9, nLeft say padc( 'Sample Reports', nLen )
   @ 10, nLeft say padc( 'Copyright (c) 1994  All Rights Reserved', nLen )
   @ 11, nLeft say padc( 'Stephen L. Woolstenhulme', nLen )
   
   @ 16, nLeft, 22, nRight box B_SOLID_TOP       // see ppptree.ch
   @ 17, nLeft say padc( 'Select A Sample Report', nLen )
   @ 18, nLeft say replicate( '', nLen )
   
   do while ! lastkey() == K_ESC
      // now you get to pick a sample report.
      @ 20, nLeft + 2 prompt ' With Memo Field '
      @ 20, col() + 2 prompt ' Directory Listing '
      @ 20, col() + 2 prompt ' Browse A File '
      menu to nChoice
      
      do case
      case nChoice == 1
         pppMemo() 
      case nChoice == 2   
         pppDir()
      case nChoice == 3
         pppBrowse()
      endcase
   enddo
   
   popscrn()
return Nil


static function PPPmemo()
   local aItem := {}
   local aColumnHead := {}
   local aPageHead := {}
   local nCount := 0
   local i := 0
   local lDone := .f.
   local cItem := ''

   pushscrn()                                    // save the screen.
   
                                                 // set the page heading.
   aPageHead := { ;
                  'BIG WIG DEALS, INC.', ;
                  'Super Important Contact List', ;
                   dtoc( date() ), ;
                  '', ;
                }

                                                 // set the column headings.
   aColumnHead := { ;
                  '=================================================================================================', ;
                  'Company                        Contact Name              Comment                                 ', ;
                  'Address #1                     Telephone                                                         ', ;
                  'Address #2                     Fax                                                               ', ;
                  'City, State, ZIP Code                                                                            ', ;
                  '============================== ========================= ========================================'  ;
                  }
   
                                                 // set report control parameters.
   pppSet( PPP_INITIALIZE )
   pppSet( PPP_TITLE, 'Demo Report Using A Memo Field' )
   pppSet( PPP_PAGEHEAD, aPageHead )
   pppSet( PPP_COLUMNHEAD, aColumnHead )
   pppSet( PPP_COLUMNSPACE, 1 )
   pppSet( PPP_FIELDWIDTH, 100 )
   pppSet( PPP_PAGECENTER, .f. )
   pppSet( PPP_PRNINIT, '' )
   pppSet( PPP_PRNRESET, '' )
   pppSet( PPP_FOOTER, .t. )
   pppSet( PPP_SOUNDS, .f. )

   if ! pppInit() == 'E'
      demoDbf()                                     // we need some data for our report.
      use pppDemo alias demo shared New             // open the data file.
   
      do while ! demo->( eof() ) .and. ! stopNow()
         aItem := {}
         aadd( aItem, { demo->COMPANY, demo->CONTACT, comment(1) } )
         aadd( aItem, { demo->ADDRESS1, padr( transform( demo->PHONE, '@R (999)-999-9999' ), len( demo->CONTACT ) ), comment(2) } )
         aadd( aItem, { demo->ADDRESS2, padr( transform( demo->FAX, '@R (999)-999-9999' ), len( demo->CONTACT ) ), comment(3) } )
         aadd( aItem, { cityline(), space( len( demo->CONTACT  ) ), comment(4) } )
         
         nCount++
         i := 5
         lDone := .f.

         do while ! lDone
            cItem := comment( i++ )
            
            if empty( cItem )
               lDone := .t.
            else
               aadd( aItem, { space( len( demo->COMPANY ) ), ;
               space( len( demo->CONTACT ) ), cItem } )
            endif

         enddo

         for i := 1 to len( aItem )
            pppLine( aItem[i] )
         next

         pppLine( '' )                           // Blank line between records.
         demo->( dbSkip(1) )
      enddo

      demo->( dbCloseArea() )
      pppLine( aTail( aColumnHead ) )
      pppLine( str( nCount, 3 ) + ' total records.' )
      pppDone()
   endif
   
   popscrn()                                     // restore the screen.
return Nil


static function PPPdir()
   local i
   local aItem := {}
   local nFiles := 0
   local nSize := 0
   
   // get data for the report.  
   local aDirectory := directory( '*.*' )
   
   // set up page and column headers.
   local aPageHead := { 'Files in ' + curdir(), date(), '' }
   
   local aColumnHead := { ;
   '===================================================', ;
   'File              Size       Date      Time    Attr', ;
   '============  ===========  ========  ========  ====' ;
   }
   
   pushscrn()                                    // push screen onto the stack.
   
   // Store the new values into the static array.  Note you need store
   // only values you want changed.  All others can be left to default
   // values.
   
   // See PPP.CH for a list of all variables you can set.
   
   pppSet( PPP_PAGEHEAD,    aPageHead   )        // set new page header.
   pppSet( PPP_COLUMNHEAD,  aColumnHead )        // set new column header.
   pppSet( PPP_COLUMNSPACE, 2 )                  // double space between columns.
   pppSet( PPP_DESPOOL, .f. )                    // gather before printing.
   pppSet( PPP_COPIES, 1 )                       // don't waste paper <g>.
   
   // Stick in code to set 80 column print on an HP.  Use chr( 18 ) on Epson.
   pppSet( PPP_PRNINIT, chr(27) + chr(38) + 'k0S' )
   
   // And a printer reset afterward.
   pppSet( PPP_PRNRESET, chr(27) + 'E' )
   
   if ! pppInit() == 'E'                         // Returns S for screen, P for
      // printer, F for file, or 
      // E for exit.  You can optionally
      // pass in S or P as a parameter
      // and the program won't pause to
      // ask.
      
      // let's sort it to make it pretty.  This slows things just a bit.
      aDirectory := asort( aDirectory,,, { | x, y | x[1] < y[1] } )
      
      for i := 1 to len( aDirectory )
         
         // build an array of items to send.  
         aItem := array( 5 )
         aItem[1] := strtran( aDirectory[ i, 1 ], '.', space( 13 - len( aDirectory[ i, 1 ] ) ) )
         aItem[2] := transform( aDirectory[ i, 2 ], '999,999,999' )
         aItem[3] := aDirectory[ i, 3 ]
         aItem[4] := aDirectory[ i, 4 ]
         aItem[5] := padc( aDirectory[ i, 5 ], 4 )
         
         // send array to be printed, with each element as a separate column.
         pppLine( aItem )
         
         // accumulate totals.
         nFiles++
         nSize += aDirectory[ i, 2 ]
         
         if stopNow()
            exit
         endif
         
      next
      
      // print underlines.
      pppLine( aColumnHead[ 3 ] )
      
      // format and print grand totals.
      aItem := { padr( ltrim( str( nFiles ) ) + ' files', 12 ), ;
      transform( nSize, '999,999,999' ) }
      pppLine( aItem )
      
      // adds final underline, lets user browse;
      // cleans up temp files and clears settings after.
      pppDone()
   endif
   
   popscrn()
return NIL


function pppBrowse()
   // no locals, no statics, no privates, nothing.
   // this report uses NOT A SINGLE VARIABLE!
   pushscrn()
   pppSet( PPP_PAGEHEAD, {} )     // don't need a page header.
   pppSet( PPP_COLUMNHEAD, {} )   // don't need a column header either.
   pppSet( PPP_FIELDWIDTH, 256 )  // longest possible line of text.
   pppSet( PPP_PAGECENTER, .f. )  // show text flush left.
   pppSet( PPP_FILTERCODES, .t. ) // let's hide these for a change.
   pppSet( PPP_PRNINIT, chr(27)+'&k2S' ) // laser compress print.

   if ! pppInit() == 'E'
      dbCreate( '____TEMP.DBF', { { 'TEXTLINE', 'C', 256, 0 } } )
      use ( '____TEMP.DBF' ) alias temp new exclusive
      append from ( 'pppdemo.prg' ) SDF
      temp->( dbGoTop() )

      do while ! temp->( eof() ) .and. ! stopNow()
         pppLine( trim( temp->TEXTLINE ) )
         temp->( dbSkip(1) )
      enddo

      temp->( dbCloseArea() )
      ferase( '____TEMP.DBF' )
      pppDone()
   endif

   popscrn()
return Nil

static function stopNow
   local nPause := inkey()
   local lRetVal := .f.
   
   if ! nPause == 0
      lRetVal := alert( ' You pressed a key.  Please select... ', ;
      { ' Continue ' , ' Quit ' }  ) == 2
   endif
   
return lRetVal

// push a screen onto the stack.
static function pushscrn()
   scrnStack( .t. )
return Nil


// pop a screen off the stack.
static function popscrn()
   scrnstack( .f. )
return Nil


// here's the screen stack handler.
static function scrnStack( lOnOff )
   static aScrn := {}
   local nLen := len( aScrn )
   
   if lOnOff                                     // push the screen onto the stack.
      aadd( aScrn, ;
      { savescreen( 0, 0, maxrow(), maxcol() ), ;
      setcolor(), row(), col(), setcursor() ;
      } )
   elseif ! empty( aScrn )                       // pop the screen back off the stack.
      restscreen( 0, 0, maxrow(), maxcol(), aScrn[ nLen, 1 ] )
      setcolor( aScrn[ nLen, 2 ] )
      setpos( aScrn[ nLen, 3 ], aScrn[ nLen, 4 ] )
      setcursor( aScrn[ nLen, 5 ] )
      aScrn := aSize( aScrn, nLen-1 )
   endif
   
return Nil


static function demoDBF()
   local aStruct := {}
   local i
   
   if ! file( 'pppDemo.dbf' )
      aadd( aStruct, { 'Contact ', 'C', 25, 0 } )
      aadd( aStruct, { 'Company ', 'C', 30, 0 } )
      aadd( aStruct, { 'Address1', 'C', 30, 0 } )
      aadd( aStruct, { 'Address2', 'C', 30, 0 } )
      aadd( aStruct, { 'City    ', 'C', 30, 0 } )
      aadd( aStruct, { 'State   ', 'C',  2, 0 } )
      aadd( aStruct, { 'ZIPCode ', 'C',  5, 0 } )
      aadd( aStruct, { 'Phone   ', 'C', 10, 0 } )
      aadd( aStruct, { 'Fax     ', 'C', 10, 0 } )
      aadd( aStruct, { 'Comment ', 'M', 10, 0 } )
      
      dbCreate( 'pppDemo.dbf', aStruct )
      use pppDemo alias demo exclusive New
      
      demo->( dbAppend() )
      demo->CONTACT := 'Stephen L. Woolstenhulme'
      demo->COMPANY := 'Rent Roll, Inc.'
      demo->ADDRESS1 := 'Rent Roll Plaza North'
      demo->ADDRESS2 := '17780 Preston Road'
      demo->CITY := 'Dallas'
      demo->STATE := 'TX'
      demo->ZIPCode := '75252'
      demo->PHONE := '2144070070'
      demo->FAX := '2144070861'
      demo->COMMENT := 'Steve Woolstenhulme is president of this company.  ' + ;
      'Rent Roll manufactures property management software written in ' + ;
      'Clipper, designed especially for the management of large multi-family ' + ;
      'apartments.'
      
      demo->( dbAppend() )
      demo->CONTACT := 'Roger L. Humiston'
      demo->COMPANY := 'Best Wholesale Company, Inc.'
      demo->ADDRESS1 := 'Suite 102'
      demo->ADDRESS2 := '17000 Dallas Parkway'
      demo->CITY := 'Dallas'
      demo->STATE := 'TX'
      demo->ZIPCode := '75248'
      demo->PHONE := '2143802299'
      demo->FAX := '2142492214'
      demo->COMMENT := 'Roger Humiston runs the Aviation Fuel Division.  ' + ;
      'We use this company to get contract into-wing jet fuel at FBOs ' + ;
      'all across the USA and at many international locations as well.  ' + ;
      'They have excellent software, written in Clipper.'
      
      demo->( dbCloseArea() )

   endif
   
return Nil


static function cityLine()
return padr( trim( demo->CITY ) + ', ' + ;
       trim( demo->STATE ) + '  ' + ;
       demo->ZIPCode, len( demo->COMPANY ) )
        

static function comment( nLine )
return MEMOLINE( demo->COMMENT, 40, nLine, 4, .t. )
   
