#include "inkey.ch"
#include "treeduce.ch"
#include "set.ch"
#include "setcurs.ch"


  // These vars are for treeducer output only
STATIC nFrame      := 0
STATIC nTray       := 0
STATIC nLineNum    := 0
STATIC nFont       := 0
STATIC nDuplex     := 0
STATIC nCopies     := 1
STATIC lGreenBar   := .T.
STATIC lWrapText   := .T.
STATIC lEscapeCode := .F.
STATIC nTabSize    := 8
STATIC nColumns    := 0
STATIC cPort       := "PRN "

STATIC cGetColor   := "n/bg"
STATIC cSayColor   := "n/w"
STATIC nLeftM, nRightM, nCenterM, nGutter, nPaperSize


// 
// Purpose: Prompt User for Treeducer Options
// Author: Kevin Maher/Steve Tyrakowski
// Copyright: SCT Associates, Inc., 1992
// 
// Section:3. Sample Programs
// Category:Sample Utilities
//
// Syntax:
//          TS_Setup( [<@nPageLength>] ) => cEscCodes
//
// Parameters:
//          <@nPageLength>  Setup will set this memory variable
//                          so that your report system can use it
//
// Returns:
//          .F. if aborted out of get screen
//          .T. if processed gets
//
// Description:
//          Will prompt the user for their preferred Treeducer options
//          for a later call to TS_Exec().  Note that the TS_Setup
//          and TS_Exec functions use file-wide statics to store the
//          user's choices for the various Treeduce options, as well as
//          their choice for port (PRN,LPT1-3).  The TS_Exec function then
//          uses those preferences when setting up the call to Treeduce().
//
//          Note that the SETting of PRINT ON, DISPLAY OFF, and PRINTER TO
//          is taken care of by TS_Exec(), as well as prompting the user
//          to turn over the paper if they are doing a manual double sided
//          printout.
//
//          The TS_Setup and TS_Exec functions are not in the TREEDUCE.LIB
//          rather, they are included in the SAMPLE directory with source
//          code because I feel that you will probably want to modify the
//          the user interface.
//
// Examples:
//          LOCAL nMaxLines := 60, cTempFile := "A1234567.890"
//          IF TS_Setup(@nMaxLines)
//            SET PRINTER TO (cTempFile)  && direct report to a file
//            DO MyReport WITH nMaxLines
//            SET PRINTER TO              && Closes Temp File
//            TS_Exec(cTempFile)
//          ENDIF
//
//
// See Also:
//        TS_Exec()
//
// Files:
//        see SAMPLE\TREESET.PRG  (not in TREEDUCE.LIB)
//
// Revisions:
//
// 
FUNCTION TS_Setup(nPageLength)
LOCAL cScreen := SaveScreen(1,4,24,78)
LOCAL GetList := {}
LOCAL lRetVal
LOCAL aMargins := TS_Margin()
LOCAL nCursor := SetCursor(SC_NORMAL)
LOCAL lConfirm := Set(_SET_CONFIRM, .T.)

  Print(1 ,5,"Ŀ")
  Print(2 ,5,"  Frame Style                         Duplex                       ")
  Print(3 ,5,"   0 No Frame  1 BoxFrame 2 T Frame      0 Single Sided            ")
  Print(4 ,5,"   3 I Frame   4 Lined    5 Lt Lined     1 Duplex Long Edge        ")
  Print(5 ,5,"Ĵ    2 Duplex Short Edge       ")
  Print(6 ,5,"  Paper Source                           3 Front Side Only         ")
  Print(7 ,5,"    0 Default Tray  1 Upper 2 Lower      4 Back Side Only          ")
  Print(8 ,5,"Ĵ    5 Prompt for each side    ")
  Print(9 ,5,"  Line #'s                           Ĵ")
  Print(10,5,"     0 No Line #'s                            Port                 ")
  Print(11,5,"     1 Filewide    3 Filewide w/wrap        Copies   PageLength    ")
  Print(12,5,"     2 Pagewide    4 Pagewide w/wrap     Wrap Text     GreenBar    ")
  Print(13,5,"   +10 Shade the Line #s              Escape Codes                 ")
  Print(14,5,"   +20 Add Vertical bar after #'s         Tab Size     1-16 ONLY   ")
  Print(15,5,"   +30 Vertical Bar and Shading       Columns     (0-8, 0 = Auto)  ")
  Print(16,5,"Ĵ")
  Print(17,5,"  Mini Font                           Paper Size                   ")
  Print(18,5,"    0 No Mini   1 Force Mini 66lines  1-Letter 2-Legal 3-Exec 4-A4 ")
  Print(19,5,"    2 Best Fit  3 Force MIni 88lines Ĵ")
  Print(20,5,"    4 Squished condensed font           Left Margin                ")
  Print(21,5,"                                       Right Margin                ")
  Print(22,5,"Ĵ Center Margin                ")
  Print(23,5,"[ESC] Cancel [F10] Process [F7] Reset        Gutter                ")
  Print(24,5,"")
        //   4567890123456789012345678901234567890123456789012345678901234567890
        //         1         2         3         4         5         6         7

	DEFAULT nPageLength TO 66
  // reset margins to defaults
  nLeftM     := aMargins[1]
  nRightM    := aMargins[2]
  nCenterM   := aMargins[3]
  nGutter    := aMargins[4]

  nPaperSize := TS_PaperSZ()

  @ 2 ,20 GET nFrame      PICTURE "9"    COLOR cGetColor VALID nFrame >= 0   .AND. nFrame <= 5
  @ 6 ,21 GET nTray       PICTURE "9"    COLOR cGetColor VALID nTray >= 0    .AND. nTray <= 2
  @ 9 ,17 GET nLineNum    PICTURE "99"   COLOR cGetColor VALID nLineNum >= 0 .AND. nLineNum <= 34
  @ 17,18 GET nFont       PICTURE "9"    COLOR cGetColor VALID nFont >= 0    .AND. nFont <= 4
  @ 2 ,52 GET nDuplex     PICTURE "9"    COLOR cGetColor VALID nDuplex >= 0  .AND. nDuplex <= 5
  @ 10,58 GET cPort       PICTURE "!!!!" COLOR cGetColor VALID cPort $ "PRN LPT1 LPT2 LPT3"
  @ 11,58 GET nCopies     PICTURE "9"    COLOR cGetColor VALID nCopies > 0
  @ 11,71 GET nPageLength PICTURE "99"   COLOR cGetColor VALID nPageLength > 0
  @ 12,58 GET lWrapText   PICTURE "Y"    COLOR cGetColor
  @ 12,71 GET lGreenBar   PICTURE "Y"    COLOR cGetColor
  @ 13,58 GET lEscapeCode PICTURE "Y"    COLOR cGetColor
  @ 14,58 GET nTabSize    PICTURE "99"   COLOR cGetColor VALID nTabSize > 0  .AND. nTabSize <= 16
  @ 15,53 GET nColumns    PICTURE "9"    COLOR cGetColor VALID nColumns >= 0 .AND. nColumns <= 4

  @ 17,56 GET nPaperSize  PICTURE "9"    COLOR cGetColor VALID nPaperSize > 0 .AND. nPaperSize < 5
  @ 20,59 GET nLeftM      PICTURE "9999" COLOR cGetColor VALID nLeftM >= 0
  @ 21,59 GET nRightM     PICTURE "9999" COLOR cGetColor VALID nRightM >= 0
  @ 22,59 GET nCenterM    PICTURE "9999" COLOR cGetColor VALID nCenterM >= 0
  @ 23,59 GET nGutter     PICTURE "9999" COLOR cGetColor VALID nGutter >= 0
	lRetVal := ReadWait(GetList)

  RestScreen(1,4,24,78,cScreen)

  Set(_SET_CONFIRM, lConfirm)
  SetCursor(nCursor)

RETURN (lRetVal)

// 
// Purpose: Call Treeducer with User's options
// Author: Kevin Maher/Steve Tyrakowski
// Copyright: SCT Associates, Inc., 1992
// 
// Section:3. Sample Programs
// Category:Sample Utilities
//
// Syntax:
//          TS_Exec( cFile ) => nil
//
// Parameters:
//          <cFile> the file that Treeducer will print.
//
// Returns:
//          nil
//
// Description:
//          Will call the Treeduce function with the parameters
//          that were setup using TS_Setup().  Note that the TS_Setup
//          and TS_Exec functions use file-wide statics to store the
//          user's choices for the various Treeduce options, as well as
//          their choice for port (PRN,LPT1-3).  The TS_Exec function then
//          uses those preferences when setting up the call to Treeduce().
//
//          Note that the SETting of PRINT ON, DISPLAY OFF, and PRINTER TO
//          is taken care of by TS_Exec(), as well as prompting the user
//          to turn over the paper if they are doing a manual double sided
//          printout.
//
// Examples:
//          LOCAL nMaxLines := 60, cTempFile := "A1234567.890"
//          IF TS_Setup(@nMaxLines)       && prompt users for options
//            SET PRINTER TO (cTempFile)  && direct report to a file
//            DO MyReport WITH nMaxLines  && create your report
//            SET PRINTER TO              && Closes Temp File
//            TS_Exec(cTempFile)          && pass the report to Treeducer
//          ENDIF
//
//
// See Also:
//          TS_Setup()
//
// Files:
//         see SAMPLE\TREESET.PRG  (not in TREEDUCE.LIB)
//
// Revisions:
//
// 
FUNCTION TS_Exec(cFile)
LOCAL bTrap := {|| Iif(InKey()==K_ESC,!(Alert("Are you sure you want to abort.",{"No","Yes"}) == 2),.T.) }
LOCAL bDisp := {|nPage,nCol|TreePrint(nPage,nCol) }
LOCAL cTreeScreen, lPrintBack

  IF (lPrintBack := (nDuplex == 5)) // 5 is print front prompt then print back
    nDuplex := TS_FRONTSIDEONLY
  ENDIF

  cTreeScreen := SaveScreen(8,14,14,66)

	DispBox(8,14,14,66,"Ŀ ",cSayColor)
  SetPos(10,20) ; DispOut("Page:",cSayColor)
  SetPos(10,32) ; DispOut("Column:",cSayColor)
  SetPos(8,30)  ; DispOut("< Send to Treeduce >", cSayColor)

  SET PRINTER TO (cPort)
  SET CONSOLE OFF
  SET PRINT ON
  Qqout( TS_PaperSZ( nPaperSize ))
  TS_Margin( nLeftM, nRightM, nCenterM, nGutter)
  Treeduce(cFile,nFrame,lGreenBar,nDuplex,nCopies,nTray,lWrapText;
	        ,lEscapeCode,"",bDisp,bTrap,nColumns, nTabSize, nFont, nLineNum)

  SET PRINT OFF
  SET CONSOLE ON
  SET PRINTER TO
  RestScreen(8,14,14,66,cTreeScreen)

  IF lPrintBack .AND. Alert("Print backside now.",{"Yes","No"}) == 1
    nDuplex  := TS_BACKSIDEONLY
    TS_Exec(cFile)
  ENDIF

RETURN (NIL)

// 
// 

STATIC FUNCTION ReadWait(aGetList)
LOCAL lRetVal := .T., lExit := .F.
LOCAL bF10 := SetKey(K_F10, {|| lExit := .T., __KeyBoard(Chr(K_CTRL_W)) })
LOCAL bF7  := SetKey(K_F7,  {|| Reset(aGetList)})

  DO WHILE !lExit
    ReadModal(aGetList)
    IF LastKey() == K_ESC
      lRetVal := .F.
			lExit := .T.
		ENDIF
  ENDDO

	SetKey(K_F7,  bF7)
	SetKey(K_F10, bF10)

RETURN (lRetVal)

// 
// 

STATIC FUNCTION Reset(aGetList)
LOCAL aMargins := TS_Margin()
  nFrame      := 0
  nTray       := 0
  nLineNum    := 0
  nFont       := 0
  nDuplex     := 0
  nCopies     := 1
  lGreenBar   := .T.
  lWrapText   := .T.
  lEscapeCode := .F.
  nTabSize    := 8
  nColumns    := 0
  cPort       := "PRN "
  nLeftM      := aMargins[1]
  nRightM     := aMargins[2]
  nCenterM    := aMargins[3]
  nGutter     := aMargins[4]

  nPaperSize  := TS_PaperSZ()

	AeVal(aGetList, {|oGet| oGet:Display()  })

RETURN (NIL)

// 
// 

STATIC FUNCTION TreePrint(nPage,nCol)
  SetPos(10,25)  ;DispOut(Transform(nPage,"999"),cSayColor)
  SetPos(10,40)  ;DispOut(Transform(nCol,"9"),cSayColor)
  DO CASE
     CASE nDuplex == TS_FRONTSIDEONLY
       SetPos(11,20)
       DispOut(Iif((nPage % 2)==0,"Skipping Back ","Printing Front"),cSayColor)
     CASE nDuplex == TS_BACKSIDEONLY
       SetPos(11,20)
       DispOut(Iif((nPage % 2)==0,"Printing Back ","Skipping Front"),cSayColor)
  ENDCASE
RETURN (NIL)

// 
// 
STATIC FUNCTION Print(nRow, nCol, xPrint, cColor)
  SetPos(nRow, nCol) ; DispOut(xPrint, Iif(cColor == NIL, cSayColor, cColor))
RETURN (NIL)

